@@ -24,7 +24,7 @@
 
 static FILE *file_in;
 static FILE *file_out;
-static void *mkshort_handle;
+static short_handle mkshort_handle;
 /* static char *deficon = NULL; */
 
 #define MYNAME "EasyGPS"
@@ -33,7 +33,7 @@ static
 arglist_t easygps_args[] = {
 /*	{"deficon", &deficon, "Default icon name", "Waypoint", 
   	ARGTYPE_STRING}, */
-	{0, 0, 0, 0 }
+	ARG_TERMINATOR
 };
 
 static void
@@ -69,7 +69,7 @@ static void
 wr_deinit(void)
 {
 	fclose(file_out);
-	mkshort_del_handle(mkshort_handle);
+	mkshort_del_handle(&mkshort_handle);
 }
 
 /*
@@ -272,5 +272,6 @@ ff_vecs_t easygps_vecs = {
 	data_read,
 	data_write,
 	NULL, 
-	easygps_args
+	easygps_args,
+	CET_CHARSET_ASCII, 0	/* CET REVIEW */
 };
