@@ -1,7 +1,7 @@
 /*
     Describe vectors containing filter operations.
  
-    Copyright (C) 2002,2004,2005 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002,2004,2005,2006 Robert Lipe, robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,8 +19,9 @@
 
  */
 
-#include <stdio.h>
 #include "defs.h"
+#include "filterdefs.h"
+#include "inifile.h"
 
 typedef struct {
 	filter_vecs_t *vec;
@@ -39,9 +40,42 @@ extern filter_vecs_t sort_vecs;
 extern filter_vecs_t stackfilt_vecs;
 extern filter_vecs_t trackfilter_vecs;
 extern filter_vecs_t discard_vecs;
+extern filter_vecs_t nuke_vecs;
+extern filter_vecs_t interpolatefilt_vecs;
 
 static
 fl_vecs_t filter_vec_list[] = {
+#if FILTERS_ENABLED
+        {
+		&arcdist_vecs,
+	        "arc",
+	        "Include Only Points Within Distance of Arc",
+	},
+	{
+		&discard_vecs,
+		"discard",
+		"Remove unreliable points with high hdop or vdop"
+	},
+	{
+		&duplicate_vecs, 
+		"duplicate",
+		"Remove Duplicates",
+	},
+	{
+		&interpolatefilt_vecs,
+		"interpolate",
+		"Interpolate between trackpoints"
+	},
+	{
+		&nuke_vecs,
+		"nuketypes",
+		"Remove all waypoints, tracks, or routes"
+	},
+	{
+		&polygon_vecs,
+		"polygon",
+		"Include Only Points Inside Polygon",
+	},
 	{
 		&position_vecs, 
 		"position",
@@ -52,31 +86,11 @@ fl_vecs_t filter_vec_list[] = {
 		"radius",
 		"Include Only Points Within Radius",
 	}, 
-	{
-		&duplicate_vecs, 
-		"duplicate",
-		"Remove Duplicates",
-	},
-        {
-		&arcdist_vecs,
-	        "arc",
-	        "Include Only Points Within Distance of Arc",
-	},
-	{
-		&polygon_vecs,
-		"polygon",
-		"Include Only Points Inside Polygon",
-	},
 	{
 		&routesimple_vecs,
 		"simplify",
 		"Simplify routes",
 	},
-	{
-		&reverse_route_vecs,
-		"reverse",
-		"Reverse stops within routes",
-	},
 	{
 		&sort_vecs,
 		"sort",
@@ -87,16 +101,17 @@ fl_vecs_t filter_vec_list[] = {
 		"stack",
 		"Save and restore waypoint lists"
 	},
+	{
+		&reverse_route_vecs,
+		"reverse",
+		"Reverse stops within routes",
+	},
 	{
 		&trackfilter_vecs,
 		"track",
 		"Manipulate track lists"
 	},
-	{
-		&discard_vecs,
-		"discard",
-		"Remove unreliable points with high hdop or vdop"
-	},
+#endif
         {
 		NULL,
 		NULL,
@@ -110,51 +125,54 @@ find_filter_vec(char *const vecname, char **opts)
 	fl_vecs_t *vec = filter_vec_list;
 	char *v = xstrdup(vecname);
 	char *svecname = strtok(v, ",");
+	int found = 0;
 
 	while (vec->vec) {
 		arglist_t *ap;
 		char *res;
 
-		if (strcmp(svecname, vec->name)) {
+		if (case_ignore_strcmp(svecname, vec->name)) {
 			vec++;
 			continue;
 		}
 
+		/* step 1: initialize by inifile or default values */
+		if (vec->vec->args) {
+			for (ap = vec->vec->args; ap->argstring; ap++) {
+				char *temp;
+				
+				temp = inifile_readstr(global_opts.inifile, vec->name, ap->argstring);
+				if (temp == NULL) temp = inifile_readstr(global_opts.inifile, "Common filter settings", ap->argstring);
+				if (temp == NULL) temp = ap->defaultvalue;
+				assign_option(vec->name, ap, temp);
+			}
+		}
+		
+		/* step 2: override settings with command-line values */
 		res = strchr(vecname, ',');
 		if (res) {
 			*opts = res+1;
 
 			if (vec->vec->args) {
 				for (ap = vec->vec->args; ap->argstring; ap++){
-					char *opt = get_option(*opts, 
-							ap->argstring);
-					if ( opts ) {
-						*ap->argval = opt;
-					}
-					else if ( ap->defaultvalue ) {
-						*ap->argval = xstrdup(
-							ap->defaultvalue);
-					}
-					else {
-						*ap->argval = NULL;
-					}
-				}
-			}
-		} else {
-			*opts = NULL;
-			if (vec->vec->args) {
-				for (ap = vec->vec->args; ap->argstring; ap++){
-					if ( ap->defaultvalue ) {
-						*ap->argval = xstrdup(
-							ap->defaultvalue);
-					}
-					else {
-						*ap->argval = NULL;
+					char *opt;
+					
+					opt = get_option(*opts, ap->argstring);
+					if ( opt ) {
+						found = 1;
+						assign_option(vec->name, ap, opt);
+						xfree(opt);
 					}
 				}
 			}
 		}
+		if (opts && opts[0] && !found) {
+			warning("'%s' is an unknown option to %s.\n", *opts, vec->name);
+		}
 
+		if (global_opts.debug_level >= 1)
+			disp_vec_options(vec->name, vec->vec->args);
+			
 		xfree(v);
 		return vec->vec;
 		
@@ -170,7 +188,10 @@ free_filter_vec( filter_vecs_t *fvec )
 	
 	if ( fvec->args ) {
 		for ( ap = fvec->args; ap->argstring; ap++) {
-			if (ap->argval && *ap->argval) xfree(*ap->argval);
+			if (ap->argval && *ap->argval) {
+				xfree(*ap->argval);
+				*ap->argval = NULL;
+			}
 		}
 	}
 }
@@ -210,6 +231,27 @@ disp_filter_vecs(void)
 	}
 }
 
+void
+disp_filter_vec( const char *vecname )
+{
+	fl_vecs_t *vec;
+	arglist_t *ap;
+
+	for (vec = filter_vec_list; vec->vec; vec++) {
+		if ( case_ignore_strcmp( vec->name, vecname )) {
+			continue;
+		}
+		printf("	%-20.20s  %-50.50s\n",
+			vec->name, vec->desc);
+		for (ap = vec->vec->args; ap && ap->argstring; ap++) {
+			if ( !(ap->argtype & ARGTYPE_HIDDEN )) 
+				printf("	  %-18.18s    %-.50s %s\n",
+				ap->argstring, ap->helpstring,
+				(ap->argtype&ARGTYPE_REQUIRED)?"(required)":"");
+		}
+	}
+}
+
 static signed int
 alpha (const void *a, const void *b)
 {
@@ -219,6 +261,24 @@ alpha (const void *a, const void *b)
         return case_ignore_strcmp(ap->desc , bp->desc);
 }
 
+static void
+disp_v1(const fl_vecs_t *vec)
+{
+	arglist_t *ap;
+	
+	for (ap = vec->vec->args; ap && ap->argstring; ap++) {
+		if ( !(ap->argtype & ARGTYPE_HIDDEN))
+			printf("option\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
+				vec->name, 
+				ap->argstring, 
+				ap->helpstring, 
+				name_option(ap->argtype),
+				ap->defaultvalue? ap->defaultvalue : "",
+				ap->minvalue? ap->minvalue : "",
+				ap->maxvalue? ap->maxvalue : "");
+	}
+}
+
 /*
  *  Display the available formats in a format that's easy to machine
  *  parse.   Typically invoked by programs like graphical wrappers to
@@ -237,8 +297,11 @@ disp_filters(int version)
 
 	switch(version) {
 	case 0:
+	case 1:
 		for (vec = filter_vec_list; vec->vec; vec++) {
 			printf("%s\t%s\n", vec->name, vec->desc);
+			if (version > 0)
+				disp_v1(vec);
 		}
 		break;
 	default:
