@@ -20,6 +20,7 @@
  */
 
 #include "defs.h"
+#if PDBFMTS_ENABLED
 #include "coldsync/palm.h"
 #include "coldsync/pdb.h"
 
@@ -48,14 +49,14 @@ struct runways {
 static FILE *file_in;
 static FILE *file_out;
 static const char *out_fname;
-struct pdb *opdb;
-struct pdb_record *opdb_rec;
+static struct pdb *opdb;
+static struct pdb_record *opdb_rec;
 static char *dbname = NULL;
 
 static
 arglist_t gpspilot_args[] = {
-        {"dbname", &dbname, "Database name", NULL, ARGTYPE_STRING},
-        {0, 0, 0, 0, 0}
+        {"dbname", &dbname, "Database name", NULL, ARGTYPE_STRING, ARG_NOMINMAX},
+        ARG_TERMINATOR
 };
 
 static void
@@ -122,7 +123,7 @@ data_read(void)
 		waypoint *wpt_tmp;
 		char *vdata;
 
-		wpt_tmp = xcalloc(sizeof(*wpt_tmp),1);
+		wpt_tmp = waypt_new();
 
 		rec = (struct record *) pdb_rec->data;
 		wpt_tmp->longitude = be_read32(&rec->longitude) / 3.6e6; 
@@ -207,7 +208,7 @@ gpspilot_writewpt(const waypoint *wpt)
         }
         vdata += strlen( vdata ) + 1;
 
-        opdb_rec = new_Record (0, 2, ct++, vdata-(char *)rec, (const ubyte *)rec);	       
+        opdb_rec = new_Record (0, 2, ct++, (uword) (vdata-(char *)rec), (const ubyte *)rec);	       
 
 	if (opdb_rec == NULL) {
 		fatal(MYNAME ": libpdb couldn't create record\n");
@@ -247,6 +248,7 @@ data_write(void)
 
 ff_vecs_t gpspilot_vecs = {
 	ff_type_file,
+	FF_CAP_RW_WPT,
 	rd_init,
 	wr_init,
 	rd_deinit,
@@ -254,5 +256,7 @@ ff_vecs_t gpspilot_vecs = {
 	data_read,
 	data_write,
 	NULL, 
-	gpspilot_args
+	gpspilot_args,
+	CET_CHARSET_ASCII, 0	/* CET-REVIEW */
 };
+#endif
