@@ -19,13 +19,15 @@
 
  */
 #include "defs.h"
+#include "filterdefs.h"
 #include "grtcirc.h"
 
+#if FILTERS_ENABLED
+
 #ifndef M_PI
 #  define M_PI 3.14159265358979323846
 #endif
 
-extern queue waypt_head;
 static route_head *cur_rte = NULL;
 
 static double pos_dist;
@@ -35,8 +37,11 @@ static char *latopt = NULL;
 static char *lonopt = NULL;
 static char *exclopt = NULL;
 static char *nosort = NULL;
+static char *maxctarg = NULL;
+static char *routename = NULL;
+static int maxct;
 
-waypoint * home_pos;
+static waypoint * home_pos;
 
 typedef struct {
 	double distance;
@@ -45,36 +50,40 @@ typedef struct {
 static
 arglist_t position_args[] = {
 	{"distance", &distopt, "Maximum positional distance",
-		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED },
+		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED, ARG_NOMINMAX },
 	{"all", &purge_duplicates, 
 		"Suppress all points close to other points", 
-		NULL, ARGTYPE_BOOL }, 
-	{0, 0, 0, 0, 0}
+		NULL, ARGTYPE_BOOL, ARG_NOMINMAX }, 
+	ARG_TERMINATOR
 };
 
 static
 arglist_t radius_args[] = {
 	{"lat", &latopt,       "Latitude for center point (D.DDDDD)",
-		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED },
+		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED, ARG_NOMINMAX },
 	{"lon", &lonopt,       "Longitude for center point (D.DDDDD)",
-		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED },
+		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED, ARG_NOMINMAX },
 	{"distance", &distopt, "Maximum distance from center",
-		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED },
+		NULL, ARGTYPE_FLOAT | ARGTYPE_REQUIRED, ARG_NOMINMAX },
 	{"exclude", &exclopt,  "Exclude points close to center",
-		NULL, ARGTYPE_BOOL },
-	{"nosort", &nosort,    "Inhibit sort by distance to center.",
-		NULL, ARGTYPE_BOOL },
-	{0, 0, 0, 0, 0}
+		NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	{"nosort", &nosort,    "Inhibit sort by distance to center",
+		NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	{"maxcount", &maxctarg,"Output no more than this number of points",
+		NULL, ARGTYPE_INT, "1", NULL },
+	{"asroute", &routename,"Put resulting waypoints in route of this name",
+		NULL, ARGTYPE_STRING, NULL, NULL },
+	ARG_TERMINATOR
 };
 
 static double
 gc_distance(double lat1, double lon1, double lat2, double lon2)
 {
 	return gcdist( 
-	    (lat1 * M_PI) / 180.0,
-	    (lon1 * M_PI) / 180.0,
-	    (lat2 * M_PI) / 180.0,
-	    (lon2 * M_PI) / 180.0
+	    RAD(lat1),
+	    RAD(lon1),
+	    RAD(lat2),
+	    RAD(lon2)
 	    );
 }
 
@@ -152,7 +161,7 @@ position_runqueue(queue *q, int nelems, int qtype)
 				   comp[i]->longitude);
 
 		/* convert radians to integer feet */
-		dist = (int)(5280*tomiles(dist));
+		dist = (int)(5280*radtomiles(dist));
 		
 		if (dist <= pos_dist) {
 			switch (qtype) {
@@ -162,6 +171,9 @@ position_runqueue(queue *q, int nelems, int qtype)
 					del = !!purge_duplicates;
 					break;
 				case trkdata:
+					track_del_wpt(cur_rte, comp[i]);
+					del = !!purge_duplicates;
+					break;
 				case rtedata:
 					route_del_wpt(cur_rte, comp[i]);
 					del = !!purge_duplicates;
@@ -178,6 +190,9 @@ position_runqueue(queue *q, int nelems, int qtype)
 					del = 0;
 					break;
 				case trkdata:
+					track_del_wpt(cur_rte, comp[i]);
+					del = !!purge_duplicates;
+					break;
 				case rtedata:
 					route_del_wpt(cur_rte, comp[i]);
 					del = 0;
@@ -196,6 +211,8 @@ position_runqueue(queue *q, int nelems, int qtype)
 				waypt_free(comp[nelems-1]);
 				break;
 			case trkdata:
+				track_del_wpt(cur_rte, comp[i]);
+				break;
 			case rtedata:
 				route_del_wpt(cur_rte, comp[i]);
 				break;
@@ -221,18 +238,24 @@ position_process_route(const route_head * rh) {
 }
 
 static void 
-position_noop(){
+position_noop_w(const waypoint *w)
+{
 }
 
-void position_process() 
+static void 
+position_noop_t(const route_head *h)
+{
+}
+
+void position_process(void) 
 {
 	int i = waypt_count();
 	
 	if (i)
 		position_runqueue(&waypt_head, i, wptdata);
 	
-	route_disp_all(position_process_route, position_noop, position_noop);
-	track_disp_all(position_process_route, position_noop, position_noop);
+	route_disp_all(position_process_route, position_noop_t, position_noop_w);
+	track_disp_all(position_process_route, position_noop_t, position_noop_w);
 }
 
 void
@@ -246,7 +269,7 @@ position_init(const char *args) {
 
 		if ((*fm == 'm') || (*fm == 'M')) {
 			 /* distance is meters */
-			pos_dist *= 3.2802;
+			 pos_dist *= 3.2802;
 		}
 	}
 }
@@ -264,6 +287,7 @@ radius_process(void)
 	waypoint ** comp;
 	int i, wc;
 	queue temp_head;
+	route_head *rte_head = NULL;
 
 	QUEUE_FOR_EACH(&waypt_head, elem, tmp) {
 		extra_data *ed;
@@ -275,7 +299,7 @@ radius_process(void)
 				   home_pos->longitude);
 
 		/* convert radians to float point statute miles */
-		dist = tomiles(dist);
+		dist = radtomiles(dist);
 
 		if ((dist >= pos_dist) == (exclopt == NULL)) {
 			waypt_del(waypointp);
@@ -312,6 +336,12 @@ radius_process(void)
 		qsort(comp, wc, sizeof(waypoint *), dist_comp);
 	}
 
+	if (routename) {
+		rte_head = route_head_alloc();
+		rte_head->rte_name = xstrdup(routename);
+		route_add_head(rte_head);
+	}
+
 	/*
 	 * The comp array is now sorted by distance.   As we run through it,
 	 * we push them back onto the master wp list, letting us pass them
@@ -319,8 +349,18 @@ radius_process(void)
  	 */
 	for (i = 0; i < wc; i++) {
 		waypoint * wp = comp[i];
-		waypt_add(wp);
+
 		xfree(wp->extra_data);
+		wp->extra_data = NULL;
+
+		if (maxctarg && i >= maxct) {
+			continue;
+		}
+		if (routename) {
+			route_add_wpt(rte_head, wp);
+		} else {
+			waypt_add(wp);
+		}
 	}
 
 	xfree(comp);
@@ -341,6 +381,12 @@ radius_init(const char *args) {
 		}
 	}
 
+	if (maxctarg) {
+		maxct = atoi(maxctarg);
+	} else {
+		maxct = 0;
+	}
+
 	home_pos = (waypoint *) xcalloc(sizeof(*home_pos), 1);
 
 	if (latopt)
@@ -370,3 +416,4 @@ filter_vecs_t radius_vecs = {
 	NULL,
 	radius_args
 };
+#endif // FILTERS_ENABLED
