@@ -21,7 +21,9 @@
  */
 
 #include "defs.h"
+#include "cet_util.h"
 #include <ctype.h>
+#include "grtcirc.h"
 
 #define MYNAME	"PSP"
 
@@ -30,7 +32,7 @@
 
 static FILE *psp_file_in;
 static FILE *psp_file_out;
-static FILE *mkshort_handle;
+static short_handle mkshort_handle;
 
 static int
 psp_fread(void *buff, size_t size, size_t members, FILE * fp) 
@@ -40,7 +42,8 @@ psp_fread(void *buff, size_t size, size_t members, FILE * fp)
     br = fread(buff, size, members, fp);
 
     if (br != members) {
-        fatal(MYNAME ": requested to read %d bytes, read %d bytes.\n", members, br);
+        fatal(MYNAME ": requested to read %ld bytes, read %ld bytes.\n", 
+                            (unsigned long) members, (unsigned long) br);
     }
 
     return (br);
@@ -154,6 +157,7 @@ valid_psp_header(char * header)
 static char *
 buffer_washer(char * buff, int buffer_len)
 {
+/* original code
     int i;
 
     for (i = 0 ; i < buffer_len - 1; i++) {
@@ -164,6 +168,11 @@ buffer_washer(char * buff, int buffer_len)
 	}
     }
 
+    return (buff);
+*/
+    char *c = cet_str_uni_to_any((const short *)buff, buffer_len >> 1, global_opts.charset);
+    strncpy(buff, c, buffer_len);
+    xfree(c);
     return (buff);
 }
 
@@ -189,7 +198,7 @@ psp_wr_init(const char *fname)
 static void
 psp_wr_deinit(void)
 {
-	mkshort_del_handle(mkshort_handle);
+	mkshort_del_handle(&mkshort_handle);
 	fclose(psp_file_out);
 }
 
@@ -231,11 +240,11 @@ psp_read(void)
 
             /* 8 bytes - latitude in radians */
 	    radians = psp_fread_double(psp_file_in);
-            lat = (radians * 180.0) / M_PI;
+            lat = DEG(radians);
 
             /* 8 bytes - longitude in radians */
 	    radians = psp_fread_double(psp_file_in);
-            lon = (radians * 180.0) / M_PI;
+            lon = DEG(radians);
 
             /* since we don't know the origin of this PSP file, we use  */
             /* the grid byte adjust longitude, if necessary, mimicing   */
@@ -349,8 +358,8 @@ psp_waypt_pr(const waypoint *wpt)
         }
 
         /* convert lat/long back to radians */
-	lat = (wpt->latitude * M_PI) / 180.0;
-        lon = (wpt->longitude * M_PI) / 180.0;
+	lat = RAD(wpt->latitude);
+        lon = RAD(wpt->longitude);
         
 	pindex++;
 	le_write16(tbuf, pindex);
@@ -466,5 +475,7 @@ ff_vecs_t psp_vecs = {
 	psp_wr_deinit,
 	psp_read,
 	psp_write,
-	NULL
+	NULL,
+	NULL,
+	CET_CHARSET_ASCII, 0	/* CET-REVIEW */
 };
