@@ -1,6 +1,8 @@
 /*
-    National Geographic Topo! Waypoint
+    National Geographic Topo! TPG file support (Waypoints/Routes)
     Contributed to gpsbabel by Alex Mottram
+
+    For Topo! version 2.x.  Routes are currently not implemented.
     
     Copyright (C) 2002 Alex Mottram, geo_alexm at cox-internet.com
 
@@ -32,10 +34,18 @@
 
 static FILE *tpg_file_in;
 static FILE *tpg_file_out;
-static void *mkshort_handle;
+static short_handle mkshort_handle;
+static char *tpg_datum_opt;
+static int tpg_datum_idx;
 
 static unsigned int waypt_out_count;
 
+static
+arglist_t tpg_args[] = {
+	{"datum", &tpg_datum_opt, "Datum (default=NAD27)", "N. America 1927 mean", ARGTYPE_STRING, ARG_NOMINMAX },
+	ARG_TERMINATOR
+};
+
 static int
 tpg_fread(void *buff, size_t size, size_t members, FILE * fp) 
 {
@@ -44,7 +54,8 @@ tpg_fread(void *buff, size_t size, size_t members, FILE * fp)
     br = fread(buff, size, members, fp);
 
     if (br != members) {
-        fatal(MYNAME ": requested to read %d bytes, read %d bytes.\n", members, br);
+        fatal(MYNAME ": requested to read %lu bytes, read %lu bytes.\n", 
+                    (unsigned long) members, (unsigned long) br);
     }
 
     return (br);
@@ -85,9 +96,19 @@ valid_tpg_header(char * header, int len)
     return memcmp(header_bytes, header, len);
 }
 
+static void
+tpg_common_init(void)
+{
+	tpg_datum_idx = GPS_Lookup_Datum_Index(tpg_datum_opt);
+	if (tpg_datum_idx < 0) {
+		fatal(MYNAME ": Datum '%s' is not recognized.\n", tpg_datum_opt);
+	}
+}
+
 static void
 tpg_rd_init(const char *fname)
 {
+	tpg_common_init();
 	tpg_file_in = xfopen(fname, "rb", MYNAME);
 }
 
@@ -100,6 +121,7 @@ tpg_rd_deinit(void)
 static void
 tpg_wr_init(const char *fname)
 {
+	tpg_common_init();
 	tpg_file_out = xfopen(fname, "wb", MYNAME);
 	mkshort_handle = mkshort_new_handle();
 	waypt_out_count = 0;
@@ -108,7 +130,7 @@ tpg_wr_init(const char *fname)
 static void
 tpg_wr_deinit(void)
 {
-	mkshort_del_handle(mkshort_handle);
+	mkshort_del_handle(&mkshort_handle);
 	fclose(tpg_file_out);
 }
 
@@ -142,9 +164,10 @@ tpg_read(void)
     	    
     	    stringsize = buff[0];
     	    
-    	    tpg_fread(&buff[0], stringsize, 1, tpg_file_in);
-
-    	    buff[stringsize] = '\0';
+	    if (stringsize)
+    	    	tpg_fread(&buff[0], stringsize, 1, tpg_file_in);
+		
+	    buff[stringsize] = '\0';
     	    
     	    wpt_tmp->shortname = xstrdup(buff);
 
@@ -162,7 +185,7 @@ tpg_read(void)
 	    
             /* 2 bytes - elevation in feet */
     	    tpg_fread(&buff[0], 2, 1, tpg_file_in);
-    	    elev = (le_read16(&buff[0]) * .3048); /* feets to meters */
+    	    elev = FEET_TO_METERS(le_read16(&buff[0]));
 
             /* convert incoming NAD27/CONUS coordinates to WGS84 */
             GPS_Math_Known_Datum_To_WGS84_M(
@@ -172,7 +195,7 @@ tpg_read(void)
                 &wpt_tmp->latitude,
                 &wpt_tmp->longitude,
                 &amt,
-                78);
+                tpg_datum_idx);
 
             wpt_tmp->altitude = elev;
             
@@ -255,14 +278,14 @@ tpg_waypt_pr(const waypoint *wpt)
                 &lat,
                 &lon,
                 &amt,
-                78);
+                tpg_datum_idx);
         
 
         /* swap the sign back *after* the datum conversion */
         lon *= -1.0;
 
         /* convert meters back to feets */
-        elev = (short int) (wpt->altitude * 3.2808);
+        elev = (short int) METERS_TO_FEET(wpt->altitude);
 
         /* 1 bytes stringsize for shortname */
         c = strlen(shortname);
@@ -364,5 +387,7 @@ ff_vecs_t tpg_vecs = {
 	tpg_wr_deinit,
 	tpg_read,
 	tpg_write,
-	NULL
+	NULL,
+	tpg_args,
+	CET_CHARSET_ASCII, 0	/* CET-REVIEW */
 };
