@@ -27,6 +27,7 @@
 #include "defs.h"
 #include "csv_util.h"
 
+#if CSVFMTS_ENABLED
 #define MYNAME	"XCSV"
 #define ISSTOKEN(a,b) (strncmp(a,b, strlen(b)) == 0)
 
@@ -43,21 +44,21 @@ static const char *intstylebuf = NULL;
 static
 arglist_t xcsv_args[] = {
 	{"style", &styleopt, "Full path to XCSV style file", NULL,
-		ARGTYPE_FILE | ARGTYPE_REQUIRED },
+		ARGTYPE_FILE | ARGTYPE_REQUIRED, ARG_NOMINMAX },
 	{"snlen", &snlenopt, "Max synthesized shortname length", NULL,
-		ARGTYPE_INT},
-	{"snwhite", &snwhiteopt, "(0/1) Allow whitespace synth. shortnames",
-		NULL, ARGTYPE_BOOL},
-	{"snupper", &snupperopt, "(0/1) UPPERCASE synth. shortnames",
-	        NULL, ARGTYPE_BOOL},
-	{"snunique", &snuniqueopt, "(0/1) Make synth. shortnames unique",
-		NULL, ARGTYPE_BOOL},
+		ARGTYPE_INT, "1", NULL},
+	{"snwhite", &snwhiteopt, "Allow whitespace synth. shortnames",
+		NULL, ARGTYPE_BOOL, ARG_NOMINMAX},
+	{"snupper", &snupperopt, "UPPERCASE synth. shortnames",
+	        NULL, ARGTYPE_BOOL, ARG_NOMINMAX},
+	{"snunique", &snuniqueopt, "Make synth. shortnames unique",
+		NULL, ARGTYPE_BOOL, ARG_NOMINMAX},
 	{"urlbase", &xcsv_urlbase, "Basename prepended to URL on output",
-	        NULL, ARGTYPE_STRING},
+	        NULL, ARGTYPE_STRING, ARG_NOMINMAX},
 	{"prefer_shortnames", &prefer_shortnames, 
 		"Use shortname instead of description", 
-		NULL, ARGTYPE_BOOL },
-	{0, 0, 0, 0, 0}
+		NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	ARG_TERMINATOR
 };
 
 /* a table of config file constants mapped to chars */
@@ -76,6 +77,7 @@ char_map_t xcsv_char_table[] = {
 	{ "SPACE",  		" "	},
 	{ "HASH",  		"#"	},
 	{ "WHITESPACE",		"\\w"	},
+	{ "PIPE",		"|"	},
 	{ NULL, 		NULL	}
 };	
 
@@ -158,7 +160,7 @@ xcsv_destroy_style(void)
         xfree(xcsv_file.extension);
 
     if (xcsv_file.mkshort_handle)
-        xfree(xcsv_file.mkshort_handle);
+        mkshort_del_handle(&xcsv_file.mkshort_handle);
 
     /* return everything to zeros */
     internal = xcsv_file.is_internal;
@@ -166,8 +168,8 @@ xcsv_destroy_style(void)
     xcsv_file.is_internal = internal;
 }
 
-static const char *
-get_char_from_constant_table(char *key)
+const char *
+xcsv_get_char_from_constant_table(char *key)
 {
     char_map_t *cm = xcsv_char_table;
 
@@ -205,7 +207,7 @@ xcsv_parse_style_line(const char *sbuff)
     if (strlen(sbuff)) {
 	if (ISSTOKEN(sbuff, "FIELD_DELIMITER")) {
 	    sp = csv_stringtrim(&sbuff[16], "\"", 1);
-	    cp = get_char_from_constant_table(sp);
+	    cp = xcsv_get_char_from_constant_table(sp);
 	    if (cp) {
 		xcsv_file.field_delimiter = xstrdup(cp);
 		xfree(sp);
@@ -232,7 +234,7 @@ xcsv_parse_style_line(const char *sbuff)
 
 	if (ISSTOKEN(sbuff, "RECORD_DELIMITER")) {
 	    sp = csv_stringtrim(&sbuff[17], "\"", 1);
-	    cp = get_char_from_constant_table(sp);
+	    cp = xcsv_get_char_from_constant_table(sp);
 	    if (cp) {
 		xcsv_file.record_delimiter = xstrdup(cp);
 		xfree(sp);
@@ -291,7 +293,7 @@ xcsv_parse_style_line(const char *sbuff)
 
 	if (ISSTOKEN(sbuff, "BADCHARS")) {
 	    sp = csv_stringtrim(&sbuff[9], "\"", 1);
-	    cp = get_char_from_constant_table(sp);
+	    cp = xcsv_get_char_from_constant_table(sp);
 
 	    if (cp) {
 	    	p = xstrdup(cp);
@@ -322,6 +324,12 @@ xcsv_parse_style_line(const char *sbuff)
 	    xcsv_epilogue_add(xstrdup(&sbuff[9]));
 	} else
 
+	if (ISSTOKEN(sbuff, "ENCODING")) {
+	    p = csv_stringtrim(&sbuff[8], "\"", 1);
+	    cet_convert_init(p, 1);
+	    xfree(p);
+	} else
+	
 	if (ISSTOKEN(sbuff, "IFIELD")) {
 	    key = val = pfc = NULL;
 	    
@@ -360,6 +368,7 @@ xcsv_parse_style_line(const char *sbuff)
 	 * change the world on ifield vs ofield format later..
 	 */
 	if (ISSTOKEN(sbuff, "OFIELD")) {
+	    int options = 0;
 	    key = val = pfc = NULL;
 
 	    s = csv_lineparse(&sbuff[6], ",", "", linecount);
@@ -379,6 +388,17 @@ xcsv_parse_style_line(const char *sbuff)
 		    /* printf conversion */
 		    pfc = csv_stringtrim(s, "\"", 1);
 		    break;
+		case 3:
+		     /* Any additional options. */
+		     if (strstr(s, "no_delim_before")) {
+			options |= OPTIONS_NODELIM;
+		     }
+		     if (strstr(s, "absolute")) {
+			options |= OPTIONS_ABSOLUTE;
+		     }
+		     if (strstr(s, "optional")) {
+			options |= OPTIONS_OPTIONAL;
+		     }
 		default:
 		    break;
 		}
@@ -386,7 +406,7 @@ xcsv_parse_style_line(const char *sbuff)
 		s = csv_lineparse(NULL, ",", "", linecount);
 	    }
 
-	    xcsv_ofield_add(key, val, pfc);
+	    xcsv_ofield_add(key, val, pfc, options);
 	}
     }
 }
@@ -420,19 +440,17 @@ xcsv_parse_style_buff(const char *sbuff)
 static void
 xcsv_read_style(const char *fname)
 {
-    char sbuff[8192];
-    FILE *fp;
+    char *sbuff;
+    gbfile *fp;
 
     xcsv_file_init();
 
-    fp = xfopen(fname, "r", MYNAME);
+    fp = gbfopen(fname, "rb", MYNAME);
 
-    do {
-        memset(sbuff, '\0', sizeof(sbuff));
-        fgets(sbuff, sizeof(sbuff), fp);
-        rtrim(sbuff);
+    while ((sbuff = gbfgetstr(fp))) {
+        sbuff = lrtrim(sbuff);
 	xcsv_parse_style_line(sbuff);
-    } while (!feof(fp));
+    } while (!gbfeof(fp));
 
     /* if we have no output fields, use input fields as output fields */
     if (xcsv_file.ofield_ct == 0) {
@@ -441,8 +459,7 @@ xcsv_read_style(const char *fname)
         xcsv_file.ofield = &xcsv_file.ifield;
         xcsv_file.ofield_ct = xcsv_file.ifield_ct;
     }
-
-    fclose(fp);
+    gbfclose(fp);
 }
 
 /*
@@ -496,17 +513,17 @@ xcsv_rd_init(const char *fname)
     }
 
     if (global_opts.masked_objective & (TRKDATAMASK|RTEDATAMASK)) {
-	warning(MYNAME "attempting to read %s as a track or route.  Converting to waypoints.\n", fname);
+	warning(MYNAME " attempt to read %s as a track or route, but this module only supports waypoints on read.  Reading as waypoints instead.\n", fname);
     }
 
-    xcsv_file.xcsvfp = xfopen(fname, "r", MYNAME);
+    xcsv_file.xcsvfp = gbfopen(fname, "r", MYNAME);
 
 }
 
 static void
 xcsv_rd_deinit(void)
 {
-    fclose(xcsv_file.xcsvfp);
+    gbfclose(xcsv_file.xcsvfp);
 
     xcsv_destroy_style();
 }
@@ -528,7 +545,7 @@ xcsv_wr_init(const char *fname)
         xcsv_read_style(styleopt);
     }
 
-    xcsv_file.xcsvfp = xfopen(fname, "w", MYNAME);
+    xcsv_file.xcsvfp = gbfopen(fname, "w", MYNAME);
     xcsv_file.fname = (char *)fname;
 
     /* set mkshort options from the command line */
@@ -555,13 +572,14 @@ xcsv_wr_init(const char *fname)
 static void
 xcsv_wr_deinit(void)
 {
-    fclose(xcsv_file.xcsvfp);
+    gbfclose(xcsv_file.xcsvfp);
 
     xcsv_destroy_style();
 }
 
 ff_vecs_t xcsv_vecs = {
     ff_type_internal,
+    FF_CAP_RW_WPT, /* This is a bit of a lie for now... */
     xcsv_rd_init,
     xcsv_wr_init,
     xcsv_rd_deinit,
@@ -569,5 +587,10 @@ ff_vecs_t xcsv_vecs = {
     xcsv_data_read,
     xcsv_data_write,
     NULL, 
-    xcsv_args
+    xcsv_args,
+    CET_CHARSET_ASCII, 0	/* CET-REVIEW */
 };
+#else
+void xcsv_read_internal_style(const char *style_buf) {}
+void xcsv_setup_internal_style(const char *style_buf) {}
+#endif //CSVFMTS_ENABLED
